@@ -23,7 +23,7 @@
  * I HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
  * ENHANCEMENTS, OR MODIFICATIONS.
  *
- * CVS: $Id: memchan.c,v 1.5 1997/05/29 12:02:08 aku Exp $
+ * CVS: $Id: memchan.c,v 1.9 1998/12/20 18:56:47 aku Exp $
  */
 
 
@@ -69,21 +69,29 @@
  * Number of milliseconds to wait between polls of channel state,
  * e.g. generation of readable/writable events.
  *
- * Relevant for Tcl 8.0 (b1) only.
+ * Relevant for Tcl 8.0 only.
  */
 
 #define DELAY (5)
 
+/* Detect Tcl 8.1 and beyond
+ */
+
+#define GT81 ((TCL_MAJOR_VERSION > 8) || \
+	      ((TCL_MAJOR_VERSION == 8) && \
+	       (TCL_MINOR_VERSION >= 1)))
+
 
 /* enable use of procedure internal to tcl */
 EXTERN void
-panic _ANSI_ARGS_ ((CONST char* format, ...));
+panic _ANSI_ARGS_ ((char* format, ...));
 
 /*
  * Forward declarations of internal procedures.
  */
 
-static int	Close _ANSI_ARGS_((ClientData instanceData, Tcl_Interp *interp));
+static int	Close _ANSI_ARGS_((ClientData instanceData,
+		   Tcl_Interp *interp));
 
 static int	Input _ANSI_ARGS_((ClientData instanceData,
 		    char *buf, int toRead, int *errorCodePtr));
@@ -94,19 +102,25 @@ static int	Output _ANSI_ARGS_((ClientData instanceData,
 static int	Seek _ANSI_ARGS_((ClientData instanceData,
 		    long offset, int mode, int *errorCodePtr));
 
+static void	WatchChannel _ANSI_ARGS_((ClientData instanceData, int mask));
+
+
+#if (TCL_MAJOR_VERSION < 8)
 static int	GetOption _ANSI_ARGS_((
 		    ClientData instanceData, char *optionName,
                     Tcl_DString *dsPtr));
 
-static void	WatchChannel _ANSI_ARGS_((ClientData instanceData, int mask));
-
-#if (TCL_MAJOR_VERSION < 8)
 static int	ChannelReady _ANSI_ARGS_((ClientData instanceData, int mask));
 static Tcl_File GetFile      _ANSI_ARGS_((ClientData instanceData, int mask));
 #else
+static int	GetOption _ANSI_ARGS_((
+		    ClientData instanceData, Tcl_Interp* interp,
+		    char *optionName, Tcl_DString *dsPtr));
+
 static void	ChannelReady _ANSI_ARGS_((ClientData instanceData));
 static int      GetFile      _ANSI_ARGS_((ClientData instanceData, int direction,
 					  ClientData* handlePtr));
+
 #endif
 
 static int      MemoryChannelCmd _ANSI_ARGS_ ((ClientData notUsed,
@@ -130,7 +144,13 @@ static Tcl_ChannelType channelType = {
 #if (TCL_MAJOR_VERSION < 8)
   ChannelReady,		/* Are there events?                             */
 #endif
+#if GT81
+  GetFile,              /* Get OS handle from the channel.               */
+  NULL                  /* Close2Proc, not available, no partial close
+			 * possible */
+#else
   GetFile               /* Get OS handle from the channel.               */
+#endif
 };
 
 
@@ -169,17 +189,21 @@ typedef struct ChannelInstance {
  *
  *------------------------------------------------------*
  */
-	/* ARGSUSED */
+/* ARGSUSED */
 static int
 Close (instanceData, interp)
-ClientData  instanceData;    /* The instance information of the channel to close */
+ClientData  instanceData;    /* The instance information of the channel to
+			      * close */
 Tcl_Interp* interp;          /* unused */
 {
   ChannelInstance* chan;
 
   chan = (ChannelInstance*) instanceData;
 
-  Tcl_Free ((char*) chan->data);
+  if (chan->data != (char*) NULL) {
+    Tcl_Free ((char*) chan->data);
+  }
+
   Tcl_Free ((char*) chan);
 
   return 0;
@@ -387,11 +411,20 @@ int*       errorCodePtr;	/* Location of error flag. */
  *------------------------------------------------------*
  */
 
+#if (TCL_MAJOR_VERSION < 8)
 static int
 GetOption (instanceData, optionName, dsPtr)
 ClientData   instanceData;	/* Channel to query */
 char*        optionName;	/* Name of reuqested option */
 Tcl_DString* dsPtr;		/* String to place the result into */
+#else
+static int
+GetOption (instanceData, interp, optionName, dsPtr)
+ClientData   instanceData;	/* Channel to query */
+Tcl_Interp*  interp;		/* Interpreter to leave error messages in */
+char*        optionName;	/* Name of reuqested option */
+Tcl_DString* dsPtr;		/* String to place the result into */
+#endif
 {
   /*
    * In-memory channels provide a channel type specific,
@@ -407,7 +440,11 @@ Tcl_DString* dsPtr;		/* String to place the result into */
 
   if ((optionName != (char*) NULL) && (0 != strcmp (optionName, "-length"))) {
     Tcl_SetErrno (EINVAL);
+#if (TCL_MAJOR_VERSION < 8)
     return TCL_ERROR;
+#else
+    return Tcl_BadChannelOption (interp, optionName, "length");
+#endif
   }
 
 
@@ -683,7 +720,6 @@ char**      argv;		/* Argument strings. */
 
   static unsigned long memCounter = 0;
 
-
   if (argc != 1) {
     Tcl_AppendResult (interp,
 		      "wrong # args: should be \"memchan\"",
@@ -691,7 +727,7 @@ char**      argv;		/* Argument strings. */
     return TCL_ERROR;
   }
 
-  instance            = (ChannelInstance*) Tcl_Alloc (sizeof (ChannelInstance));
+  instance = (ChannelInstance*) Tcl_Alloc (sizeof (ChannelInstance));
   instance->rwLoc     = 0;
   instance->allocated = 0;
   instance->used      = 0;
@@ -710,11 +746,11 @@ char**      argv;		/* Argument strings. */
   instance->timer     = (Tcl_TimerToken) NULL;
 #endif
 
-
   Tcl_RegisterChannel  (interp, chan);
   Tcl_SetChannelOption (interp, chan, "-buffering", "none");
   Tcl_SetChannelOption (interp, chan, "-blocking",  "0");
   Tcl_AppendResult     (interp, channelName, (char*) NULL);
+
   return TCL_OK;
 }
 
